<!-- XSLT script to extract document for class/function for swig docstring
If you have xsltproc you could use:
xsltproc swig.xsl index.xml > swig_doc.i
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text"/>
<xsl:template match="/">
<!-- process each compound one by one -->
<xsl:for-each select="doxygenindex/compound">
<xsl:apply-templates select="document( concat( @refid, '.xml' ) )/*" />
</xsl:for-each>
</xsl:template>

<xsl:template match="doxygen">
<!-- for each class, output brief and detailed description -->
<xsl:for-each select="compounddef[@kind='class']|compounddef[@kind='namespace']"> <xsl:text>%feature("docstring") </xsl:text> 
<xsl:value-of select="compoundname"/>
<xsl:text> " &#10;</xsl:text>
<xsl:apply-templates select="briefdescription"/><xsl:text>
</xsl:text>
<xsl:apply-templates select="detaileddescription"/>
<xsl:text> see also: </xsl:text>
<xsl:value-of select="includes"/>
<xsl:text>";&#10;&#10;</xsl:text>

<!-- output for each function individually -->
<xsl:for-each select="*/memberdef[@kind='function' and not(starts-with(name,'operator'))]"> <xsl:text>%feature("docstring") </xsl:text><xsl:value-of select="../../compoundname"/>::<xsl:value-of select="name"/> 
<xsl:text> "&#10;</xsl:text>
<xsl:value-of select="definition"/> <xsl:apply-templates 
select="argsstring"/>
<xsl:text>
</xsl:text><xsl:apply-templates select="briefdescription"/><xsl:text>
</xsl:text><xsl:apply-templates select="detaileddescription"/>
<xsl:text>";&#10;</xsl:text>
</xsl:for-each>
</xsl:for-each> </xsl:template> 

<!-- Call the recursive text cleaner with the entire content of
the descriptive elements -->
<xsl:template match="briefdescription|detaileddescription|argsstring">
<xsl:call-template name="CleanQuotes"><xsl:with-param
name="body" select="."></xsl:with-param>
</xsl:call-template>
</xsl:template>

<!-- recursive cleaner, matches string to first quote then recurses to process 
the
rest. Note how it uses a When to test if the termination condition of no
quotes has been reached
-->
<xsl:template name="CleanQuotes"><xsl:param name="body"/>
<xsl:choose>
<xsl:when test="contains($body, '&quot;')">
<xsl:value-of select="substring-before($body, '&quot;')"/>\&quot;<xsl:call-template 
name="CleanQuotes">
<xsl:with-param name="body" 
select="substring-after($body,'&quot;')"></xsl:with-param></xsl:call-template>
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="$body"/>
<!-- finished recursing -->
</xsl:otherwise>
</xsl:choose>
</xsl:template>

</xsl:stylesheet>

